#include "linklist.h"

// Allocate()
// desc: function to allocate memory for a node
// returns a freshly allocated node
node_t* Allocate()
{
	node_t *newNode;		// our new node

	// request memory for our node
	newNode = (node_t*)malloc(sizeof(node_t));

	// error checking
	if (newNode == NULL)
		printf("Error: Unable to allocate new node.\n");
	else
		newNode->next = NULL;	// allocation succeeded, set the next link to NULL

	return newNode;
}

// Initialize()
// desc: function created specifically for the head of the list to set it NULL
void Initialize(node_t **ptr)
{
	*ptr = NULL;
}

// Insert()
// desc: function to insert a new node at the front of a list
void InsertFront(node_t **head, node_t newRecord)
{
	node_t*  newNode;		// pointer to a new node
	newNode = Allocate();

	if (newNode == NULL)
	{
		printf("Error: Not enough memory to allocate node.\n");
		return;
	}

	// fill the node with data
	newNode->id = newRecord.id;

	// insert at the front of the list
	newNode->next = *head;

	// move the head of the list to the new node
	*head = newNode;	
}

// DisplayList()
// desc: a traverse list example where we display the data in the list
void DisplayList(node_t *head)
{
	node_t *current;		// our current node (position)

	// begin at the head of the list
	current = head;

	// loop until done
	while (current != NULL)
	{
		printf("ID = %d\n", current->id);
		current = current->next;
	}
}

// DeleteNode()
// desc: function to delete a node with the data specified by keyRecord
void DeleteNode(node_t **head, node_t keyRecord)
{
	node_t *delNode;		// node to delete
	node_t *previous;		// node before the deleted node
		
	// find our node to delete
	delNode = Find(*head, keyRecord);

	// if desired record is not in the list, exit the function
	if (delNode == NULL)
	{
		printf("Record not found.\n");
		return;
	}

	if (delNode == *head)
	{
		// first node in the list, but not the only node
		// move the head to the second node in the list
		*head = delNode->next;
		free((void*)delNode);
	}
	else
	{
		// any other case
		previous = *head;

		// search through the list for the node before our deleted node
		while (previous->next != delNode)
		{
			previous = previous->next;
		}

		// link the previous node to the node after our deleted node
		previous->next = delNode->next;

		if (delNode != NULL)
		{
			free((void*)delNode);	// free the memory
			delNode = NULL;	
		}
	}
}

// Find()
// desc: function that returns a pointer to the data being searched for in keyRecord
// returns NULL if nothing is found
node_t *Find(node_t *head, node_t keyRecord)
{
	node_t *current;
	bool found;

	current = head;
	found = false;

	while ((current != NULL) && (!found))		// traverse the list until the key is found
	{											// or we reach the end
		if (current->id == keyRecord.id)
			found = true;
		else
			current = current->next;
	}

	return current;
}